/*******************************************************************************

File:   AlimHardlib.h
Author : Luca Tanganelli (shinshi)
Version : 1.0

Created on Date : 31/03/2013
Last update     : 10/08/2013

CopyRight 2013 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Luca Tanganelli (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgment with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.


********************************************************
PURPOSES
********************************************************


 Libreria di impostazione ed inizializzazione dell'Hardware dell'Alimentatore
 Digitale da Banco

*******************************************************************************/

#include <p18cxxx.h>
#include <pwm.h>
#include <timers.h>

#ifndef ALIMHARDLIB_H
#define	ALIMHARDLIB_H

//Uscite digitali
#define BUZZ        LATCbits.LATC0
#define FAN         LATCbits.LATC7
#define ATTIVA      LATEbits.LATE1
#define SCARICA     LATEbits.LATE2

//Input digitali
#define SW_LEFT     PORTBbits.RB4
#define SW_DOWN     PORTBbits.RB5
#define SW_RIGHT    PORTBbits.RB6
#define SW_ON       PORTBbits.RB7
#define USB_DETECT  PORTCbits.RC6
#define LIMITAZIONE PORTAbits.RA4

//Analog input
#define READVOLT    0 //AN0
#define READAMP     1 //AN1
#define POTVOLT     2 //AN2
#define POTAMP      3 //AN3
#define TEMP        4 //AN4
#define AUX         5 //AN5


//Periodo del PWM frequenza 20KHz
/*Calcolo:
 *            Fosc                      20.000.000
 * PERIOD = _____________________ -1 = ____________ -1 = 249
 *          4*Fpwm*TMR2prescaler        4*20.000*1
*/
#define PERIOD 249

#define TURN_ON     1
#define TURN_OFF    0

//Temperature ranges
// Once TEMP_MAX is reached, the system should be shouted down.
// Reaching TEMP_WARNING the fan should be turned ON.
#define TEMP_MAX 60
#define TEMP_WARNING 30
#define TEMP_HYSTERESIS 6

//System Status
#define OVER_VOLTAGE 	0b00000001
#define UNDER_VOLTAGE 	0b00000010
#define OVER_CURRENT	0b00000100
#define OVER_TEMP	0b00001000
#define FAN_RUN 	0b00010000
#define NOT_DEFINED_2 	0b00100000
#define NOT_DEFINED_3 	0b01000000
#define NOT_DEFINED_4 	0b10000000


//**************************************************
// Function Prototypes
//**************************************************

/**
 * Questa funzione inizializza la scheda e l'hardware.
 * Vengono settati tutti i PIN del PIC.
 * Deve essere richiamata solo una volta all'inizio del main ()
 *
 * @param
 *
 * @return
 */
void initialization (void);

/**
 * Funzione di lettura degli ADC.
 *
 *
 *
 * @param channel canale del ADC del PIC. Vedi lista costanti degli input Analogici.
 *
 * @param redundancy valore che messo ad esponente di 2 ci da il numero di campionamenti
 * di cui fare la media. Esempio se  3 verranno eseguiti 8 campinamenti e fatta la media.
 *
 * @return Valore RAW letto in ingresso analogico selezionato
 */
int ADC_read (char channel, char redundancy);

/**
 * Funzione di GESTIONE VENTOLA.
 *
 * @param
 *
 * @return
 *
 */
void tempcheck (void);

#endif	/* ALIMHARDLIB_H */

