/*******************************************************************************

Author : Mauro Laurenti
Version : 1.4

Created on Date : 04/09/2006
Last update     : 30/04/2013

CopyRight 2006-2013 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgment with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.


********************************************************
PURPOSES
********************************************************


 This library contains all the functions that are handy for controlling 			
 an LCD with a 44780 Hitachi controller.							
 	
 To use these functions for your pourpose you must set up properly 
 the LCD constants and the pins of the PORT/s that you will use.
 You can use any pin you want!

*******************************************************************************/

#ifndef LCD_44780_H
#define LCD_44780_H

#ifdef __XC8
    #include <xc.h>
    #include <stdlib.h>
#endif

#include <delay.h>


#ifndef __XC8
    #include <ctype.h>
    #include <p18cxxx.h>
#endif

//**************************************************
// LCD constants
// All the following pins must be set as output
//**************************************************

/**
 * This value are used as default ones
 *
 */
#ifndef LCD_DEFAULT

        #ifdef _PIC18
            #warning LCD_D0 has been not defined, LATDbits.LATD4 will be used
            #warning LCD_D1 has been not defined, LATDbits.LATD5 will be used
            #warning LCD_D2 has been not defined, LATDbits.LATD6 will be used
            #warning LCD_D3 has been not defined, LATDbits.LATD7 will be used
            #warning LCD_RS has been not defined, LATDbits.LATD2 will be used
            #warning LCD_E has been not defined, LATDbits.LATD3 will be used
            #warning LCD_RW has been not defined, LATDbits.LATD1 will be used
            #warning LCD_LED has been not defined, LATCbits.LATC1 will be used
        #endif

        #ifndef _PIC18
            #warning LCD_D0 has been not defined, PORTDbits.RD4 will be used
            #warning LCD_D1 has been not defined, PORTDbits.RD5 will be used
            #warning LCD_D2 has been not defined, PORTDbits.RD6 will be used
            #warning LCD_D3 has been not defined, PORTDbits.RD7 will be used
            #warning LCD_RS has been not defined, PORTDbits.RD2 will be used
            #warning LCD_E has been not defined, PORTDbits.RD3 will be used
            #warning LCD_RW has been not defined, PORTDbits.RD1 will be used
            #warning LCD_LED has been not defined, PORTCbits.RC1 will be used
        #endif
	
#endif
        #ifdef _PIC18
            #define LCD_D0 LATDbits.LATD4
            #define LCD_D1 LATDbits.LATD5
            #define LCD_D2 LATDbits.LATD6
            #define LCD_D3 LATDbits.LATD7
            #define LCD_RS LATDbits.LATD2
            #define LCD_E  LATDbits.LATD3
            #define LCD_RW LATDbits.LATD1
            #define LCD_LED LATCbits.LATC1
        #endif

        #ifndef _PIC18
            #define LCD_D0 PORTDbits.RD4
            #define LCD_D1 PORTDbits.RD5
            #define LCD_D2 PORTDbits.RD6
            #define LCD_D3 PORTDbits.RD7
            #define LCD_RS PORTDbits.RD2
            #define LCD_E  PORTDbits.RD3
            #define LCD_RW PORTDbits.RD1
            //#define LCD_LED PORTCbits.RC1 la retroilluminazione del display  fissa e RC1  gi impegnato
        #endif
	


//**************************************************
//               Constant Definitions

#define LEFT 0
#define RIGHT 1

#define TURN_ON_LED_LCD 1
#define TURN_OFF_LED_LCD 0

#define TURN_ON_CURSOR 1
#define TURN_OFF_CURSOR 0

#define BLINKING_ON 1
#define BLINKING_OFF 0

//**************************************************


/**
 * This function generates the Enable pulse.
 *
 *
 * @warning This function is a Private one. It should not be used by the user.
 */
void enable_pulse_LCD (void);
#define Epulse enable_pulse_LCD


/**
 * This function generates the Enable pulse.
 *
 *
 * @param bit_0 bit 0 of the data bus (4 bit modality)
 *
 * @param bit_1 bit 1 of the data bus (4 bit modality)
 *
 * @param bit_2 bit 2 of the data bus (4 bit modality)
 *
 * @param bit_3 bit 3 of the data bus (4 bit modality)
 *
 * @warning This function is a Private one. It should not be used by the user.
 */
void send_command_LCD (unsigned char bit_3, unsigned char bit_2, unsigned char bit_1, unsigned char bit_0);
#define SendCommand send_command_LCD


/**
 * This function locates the cursor at home location. First line first character.
 *
 */
void home_LCD(void);
#define HomeLCD home_LCD


/**
 * This function shifts the LCD screen on the left or rigt.
 *
 * @param shift Specify where the shift should be [LEFT, RIGHT].
 *
 * @param number_of_shift Specify the number of times the shift is executed.
 *
 * @note Use the constants LEFT, RIGHT to specify the direction
 *       to ensure the compatibility with LCD_44780_I2C library.
 */
void shift_LCD(unsigned char shift, unsigned char number_of_shift);
#define ShiftLCD shift_LCD


/**
 * This function shifts the LCD cursor on the left or rigt. The position of the cursor is where the
 * next writting will be performed.
 *
 * @param shift Specify where the shift should be [LEFT, RIGHT].
 *
 * @param number_of_shift Specify the number of times the shift is executed.
 *
 * @note Use the constants LEFT, RIGHT to specify the direction
 *       to ensure the compatibility with LCD_44780_I2C library.
 */
void shift_cursor_LCD(unsigned char shift,unsigned char number_of_shift);
#define ShiftCursorLCD shift_cursor_LCD


/**
 * This function locates the LCD cursor on the selected line. 
 *
 * @param line Specify the number of the line [1,2,3,4]
 *
 * @warning It might not work with all the LCD Dipslays. Tested on 20x4 16x2 LCD displays.
 */
void goto_line_LCD (unsigned char line);
#define GotoLineLCD goto_line_LCD


/**
 * This function locates the LCD cursor to an arbitrary X Y location.
 * Y represents the line number from top.
 *
 * @param x It Specifies horizontal position
 *
 * @param y It Specifies vertical position (line number)
 *
 * @warning It might not work with all the LCD Dipslays. Tested on 20x4 16x2 LCD displays.
 */
void goto_xy_LCD (unsigned char x, unsigned char y);


/**
 * This function writes a char to the LCD display. (e.g WriteCharLCD ('a'); )
 *
 * @param value Specify the character to be sent [0-128 ASCII code].
 *
 * @note The function accept ASCII integer or constants 'a'..'z'.
 */
void write_char_LCD (unsigned char value);
#define WriteCharLCD write_char_LCD


/**
 * This function writes a const string to the LCD display, e.g write_message_LCD("Hello"); .
 *
 * @param buffer Is a const string written in rom (is not an array).
 *
 * @note The function accepts strings written in rom (constant), it does not accept array.
 *       To write arrays refers the write_string_LCD () function.
 *
 */
#ifndef __XC8
void write_message_LCD(const rom unsigned char *buffer);
#endif

#ifdef __XC8
void write_message_LCD(const unsigned char *buffer);
#endif

#define WriteStringLCD write_message_LCD


/**
 * This function writes an array of char to the LCD display, terminated with /0 .
 *
 * @param buffer It is an array of char terminated with /0 .
 *
 * @note The function accepts strings  within an array. To use const array within flash use write_message_LCD ().
 *
 */
void write_string_LCD(unsigned char *buffer);
#define WriteVarLCD write_string_LCD


/**
 * This function writes an integer to the LCD display. The integer is converted to string.
 *
 * @param value It is integer that must be written to the LCD diplay.
 *
 * @param numeber_of_digits It specifies the number of shown digit [0-5].
*                    0: Left Justified
*                    1-5: Right Justified with n digit
 *
 * @note If you set a number of digit less than required the digit will be lost starting from the less
 *       significant digit. Minus is like a digit.
 *
 */
void write_integer_LCD(int value, unsigned char number_of_digits);
#define WriteIntLCD write_integer_LCD


/**
 * This Function cleans the LCD display.
 *
 */
void clear_LCD (void);
#define ClearLCD clear_LCD


/**
 * This function controls the cursor option (blinking, active)
 *
 * @param active Activate the cursor, showing it or not [TURN_ON_CURSOR, TURN_OFF_CURSOR]
 *
 * @param blinking  Let the cursor blink [BLINKING_ON,BLINKING_OFF]
 *
 * @note You must use the constant TURN_ON_CURSOR, TURN_OFF_CURSOR, BLINKING_ON, BLINKING_OFF
 *       to ensure the compatibility with LCD_44780_I2C library.
 *
 */
void cursor_LCD(unsigned char active, unsigned char blinking);
#define CursorLCD cursor_LCD


/**
 * This function controls the backligth LED.
 *
 * @param active Activate the LED backlight [TURN_ON_LED_LCD, TURN_OFF_LED_LCD]
 *
 * @param blinking  Let the cursor blink [BLINKING_ON,BLINKING_OFF]
 *
 * @note You must use the constant TURN_ON_LED_LCD, TURN_OFF_LED_LCD
 *       to ensure the compatibility with LCD_44780_I2C library.
 */
//void backlight_LCD(unsigned char active); Funzione non utilizzata
//#define BacklightLCD backlight_LCD


/**
 * This funnction initializes the LCD to work in 4 bit modality.
 *
 * @param quartz_frequency Quartz freq. expressed in MHz used to run the CPU.
 *
 *
 * @note You must properly set the microcontroller pins using the TRISx registers
 *       The library requires the delay library to be included.
 */
void  initialize_LCD(unsigned char quartz_frequency);
#define OpenLCD initialize_LCD

#endif
