/*******************************************************************************

File:   main.c
Author : Luca Tanganelli (shinshi)
Version : 1.0

Created on Date : 31/03/2013
Last update     : 10/08/2013

CopyRight 2013 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Luca Tanganelli (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgment with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*******************************************************************************/

#define LCD_DEFAULT
#include "LCD_44780.h"
//#include "LCD_44780.c"
#include "delay.h"
//#include "delay.c"
#include "AlimHardlib.h"
//#include "AlimHardlib.c"
#include "AlimFunc.h"
//#include "AlimFunc.c"


#pragma config FOSC = HS
#pragma config WDT = OFF
#pragma config LVP = OFF
#pragma config PBADEN = OFF
#pragma config CCP2MX = ON

//OSC = HS  	Impostato per lavorare ad alta frequenza
//WDT = OFF		Disabilito il watchdog timer
//LVP = OFF 	Disabilito programmazione LVP
//PBADEN = OFF	Disabilito gli ingressi analogici
//CPP2 = ON 	Il modulo CCP  posto su RC1

void New_High_Priority_ISR_Code();
void New_Low_Priority_ISR_Code();
//*************************************************************
// Definizione nuovi indirizzi
//*************************************************************
#define REMAPPED_RESET_VECTOR_ADDRESS 0x1000
#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS 0x1008
#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS 0x1018
//Vedi il file c018i.c
extern void _startup (void);
//*************************************************************
// Rimappatura Interrupt Service Routine
//*************************************************************
#pragma code REMAPPED_RESET_VECTOR = REMAPPED_RESET_VECTOR_ADDRESS
void _reset (void){
_asm goto _startup _endasm
}
#pragma code REMAPPED_HIGH_INTERRUPT_VECTOR = REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS
void Remapped_High_ISR (void) {
_asm goto New_High_Priority_ISR_Code _endasm
}
#pragma code REMAPPED_LOW_INTERRUPT_VECTOR = REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS
void Remapped_Low_ISR (void) {
_asm goto New_Low_Priority_ISR_Code _endasm
}
//*************************************************************
// Standard Interrupt Service Routine
//*************************************************************
#pragma code HIGH_INTERRUPT_VECTOR = 0x08
void High_ISR (void) {
_asm goto REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS _endasm
}
#pragma code LOW_INTERRUPT_VECTOR = 0x18
void Low_ISR (void) {
_asm goto REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS _endasm
}
#pragma code
//*************************************************************
// Nuovi Interrupt Service Routines
//*************************************************************
#pragma interrupt New_High_Priority_ISR_Code
void New_High_Priority_ISR_Code() {
// Gestione Interrupt alta Priorita'

    // Indice per il ciclo di pausa
    int i;
    // Controllo che l'interrupt sia stato generato da PORTB
    if (INTCONbits.RBIF == 1 ) {
        //pausa filtraggio spike
        for (i=0; i<30000; i++){
        }
        // Controllo la pressione di SW1
        if (SW_ON == 0) {
            // Inverto lo stato dell'alimentatore in uscita
            ATTIVA = ~ATTIVA;
        }
        //Controllo la pressione di SW2
        if (SW_DOWN == 0) {
            //Attivo la scarica
            SCARICA = ~SCARICA;
        }

        // Resetto il flag d'interrupt per permettere nuove interruzioni
        INTCONbits.RBIF = 0;
    }
}
#pragma interruptlow New_Low_Priority_ISR_Code
void New_Low_Priority_ISR_Code(){
// Gestione Interrupt bassa Priorita'
}

//*************************************************************
//              Funzione Main ()
//*************************************************************

void main (void){


    int raw_potvolt; //lettura ADC Potenziometro Volt
    int raw_potamp;  //lettura ADC Potenziometro Ampere
    int potvolt; //Valore convertito in centesimi di volt del Potenziometro
    int potamp; //Valore convertito in decimi di ampere del Potenziometro
    int readvolt; //Valore convertito in centesimi di volt dell'uscita
    int readamp; //Valore convertito in decimi di ampere dell'uscita
    int pwm_volt; //Valore in uscita al DAC PWM dei Volt
    int pwm_amp;  //Valore in Uscita al DAC PWM degli Ampere
    int old_potvolt; //Valore potenziometro precedente
    int old_potamp; //Valore precedente potenziometro ampere
    char comp_volt=0; //compensazione fine volt
    char comp_amp=0;  //compensazione fine ampere
    

    //CONFIGURAZIONE SCHEDA
    initialization ();
    // Abilita i resistori di pull-up sulla PORTB
    INTCON2bits.RBPU=0;
    // Abilito le interruzioni su PORTB
    INTCONbits.RBIE = 1;
    // Abilito modalit compatibile (di default vale gi 0)
    RCONbits.IPEN = 0;
    // Abilito l'interrupt globale
    INTCONbits.GIE = 1;
    // Abilito l'interrupt periferiche
    INTCONbits.PEIE = 1 ;
    // Inizializzo il display LCD con quarzo a 20MHz
    initialize_LCD (20);




    while (1){
        tempcheck ();

        //se il potenziometro  stabile esegue la compensazione dei volt se
        //non siamo in limitazione di corrente
        if (potvolt == old_potvolt && LIMITAZIONE == 0){
            comp_volt = offsetting (potvolt,readvolt,comp_volt);
        }
        //se il potenziometro  stabile esegue la compensazione degli ampere se
        //siamo in limitazione di corrente
        if (potamp == old_potamp && LIMITAZIONE == 1){
            comp_amp = offsetting (potamp,readamp,comp_amp);
        }

        //Allineamento tra l'impostazione del potenziometro e l'effettiva uscita convertitore
        pwm_volt = pot2pwm(raw_potvolt, comp_volt);
        pwm_amp = pot2pwm(raw_potamp, comp_amp);

        //Settaggio PWM
        SetDCPWM1 (pwm_volt);
        SetDCPWM2 (pwm_amp);

        //memorizzo il valore attuale del potenziometro per vedere se varia.
        old_potvolt = potvolt;
        old_potamp = potamp;

        //Lettura ADC e scrittura nell'LCD
        //Prima riga dedicata alla tensione
        write_message_LCD (" V: ");
        //Legge, convrte e arrotonda l'ADC della tensione
        //Il risultato  espresso in decimi di volt
        readvolt = rounding ((ADC_read (READVOLT, 3) * 30),2);
        //Viene scritto il valore nel display espresso in volt aggiungendo un decimale
        write_decimal_LCD (readvolt, 4, 1);
        //Se la limitazione di corente non  attiva scrive una frecia.
        if (LIMITAZIONE == 0){
            write_char_LCD (126);
        }else{
            write_message_LCD (" ");
        }
        //Tra parentesi scrive il valore impostato con i potenziometri.
        write_message_LCD ("(");
        //memorizza il valore grezzo dell'ADC
        raw_potvolt = ADC_read (POTVOLT, 3);
        //Converte e arrotonda al decimo di volt
        potvolt = rounding ((raw_potvolt * 30),2);
        //Scrive il valore in volt con un decimale
        write_decimal_LCD (potvolt, 4, 1);
        write_message_LCD (")");

        goto_line_LCD (2);

        //Seconda riga dedicata alla corrente
        write_message_LCD (" A: ");
        //Legge, converte e arrotonda l'ADC della corrente
        //Il risultato  espresso in centesimi di ampere
        readamp = rounding ((ADC_read (READAMP, 3) * 5),1);
        //Viene scritto il valore nel display espresso in ampere aggiungendo due decimali
        write_decimal_LCD (readamp, 4, 2);
        //Se la limitazione di corente  attiva scrive una frecia.
        if (LIMITAZIONE == 1){
            write_char_LCD (126);
        }else{
            write_message_LCD (" ");
        }
        //Tra parentesi scrive il valore impostato con i potenziometri.
        write_message_LCD ("(");
        //memorizza il valore grezzo dell'ADC
        raw_potamp = ADC_read (POTAMP, 3);
        //Converte e arrotonda al centesimo di ampere
        potamp = rounding ((raw_potamp * 5),1);
        //Scrive nel display il valore in ampere con 2 decimali
        write_decimal_LCD (potamp, 4, 2);
        write_message_LCD (")");

        home_LCD ();


    }


}